@@ -39,19 +39,26 @@ get_objects <- function(type, selection = NULL, sample_prior = F, design = NULL,
   return(out)
 }
 
-add_prior_names <- function(prior, design){
-  pnames <- names(sampled_p_vector(design))
+add_prior_names <- function(prior, design, ...){
+  dots <- list(...)
+  pnames <- names(sampled_pars(design))
+  nuisance <- is.element(seq_len(length(pnames)), unique(c(dots$nuisance, dots$nuisance_non_hyper)))
+  # This might be a weird place to set the nuisance parameters in the prior,
+  # but it's the most efficient for now
   for(pri in names(prior)){
     if(is.null(dim(prior[[pri]]))){
       if(length(pnames) == length(prior[[pri]])){
-        names(prior[[pri]]) <- pnames
+        prior[[pri]] <- prior[[pri]][!nuisance]
+        names(prior[[pri]]) <- pnames[!nuisance]
       }
     } else{
       if(length(pnames) == nrow(prior[[pri]])){
-        rownames(prior[[pri]]) <- pnames
+        prior[[pri]] <- prior[[pri]][!nuisance,]
+        rownames(prior[[pri]]) <- pnames[!nuisance]
       }
       if(length(pnames) == ncol(prior[[pri]])){
-        colnames(prior[[pri]]) <- pnames
+        prior[[pri]] <- prior[[pri]][,!nuisance]
+        colnames(prior[[pri]]) <- pnames[!nuisance]
       }
     }
   }
@@ -78,7 +85,7 @@ get_objects_diag <- function(selection, sample_prior, return_prior, design = NUL
       Sigma = 'Group-level covariance matrix'
     )
     if(!is.null(list(...)$return_info))return(prior)
-    prior$prior <- add_prior_names(prior$prior, design)
+    prior$prior <- add_prior_names(prior$prior, design, ...)
     return(prior)
   } else{
     if(!selection %in% acc_selection) stop(paste0("selection must be in : ", paste(acc_selection, collapse = ", ")))
@@ -91,7 +98,8 @@ get_objects_diag <- function(selection, sample_prior, return_prior, design = NUL
       } else{
         sampler <- list(list(samples = get_prior_diag(prior = prior, design = design, selection = selection,N = N)))
       }
-      attr(sampler, "design_list") <- list(design)
+      sampler[[1]]$prior <- prior
+      class(sampler) <- "emc"
       return(sampler)
     }
     idx <- get_idx(sampler, stage)
@@ -119,7 +127,7 @@ get_objects_diag_gamma <- function(selection, sample_prior, return_prior, design
       Sigma = 'Group-level covariance matrix'
     )
     if(!is.null(list(...)$return_info)) return(prior[c("types", "type_descriptions", "descriptions")])
-    prior$prior <- add_prior_names(prior$prior, design)
+    prior$prior <- add_prior_names(prior$prior, design, ...)
     return(prior)
   } else{
     if(!selection %in% acc_selection) stop(paste0("selection must be in : ", paste(acc_selection, collapse = ", ")))
@@ -132,7 +140,8 @@ get_objects_diag_gamma <- function(selection, sample_prior, return_prior, design
       } else{
         sampler <- list(list(samples = get_prior_diag_gamma(prior = prior, design = design, selection = selection,N = N)))
       }
-      attr(sampler, "design_list") <- list(design)
+      sampler[[1]]$prior <- prior
+      class(sampler) <- "emc"
       return(sampler)
     }
     idx <- get_idx(sampler, stage)
@@ -161,7 +170,7 @@ get_objects_standard <- function(selection, sample_prior, return_prior, design =
       Sigma = 'Group-level covariance matrix'
     )
     if(!is.null(list(...)$return_info)) return(prior[c("types", "type_descriptions", "descriptions")])
-    prior$prior <- add_prior_names(prior$prior, design)
+    prior$prior <- add_prior_names(prior$prior, design, ...)
     return(prior)
   } else{
     if(!selection %in% acc_selection) stop(paste0("selection must be in : ", paste(acc_selection, collapse = ", ")))
@@ -174,7 +183,8 @@ get_objects_standard <- function(selection, sample_prior, return_prior, design =
       } else{
         sampler <- list(list(samples = get_prior_standard(prior = prior, design = design, selection = selection,N = N)))
       }
-      attr(sampler, "design_list") <- list(design)
+      sampler[[1]]$prior <- prior
+      class(sampler) <- "emc"
       return(sampler)
     }
     idx <- get_idx(sampler, stage)
@@ -187,6 +197,7 @@ get_idx <- function(sampler, stage){
     dims <- dim(sampler[[1]][[1]][[1]])
     idx <- 1:(dims[length(dims)])
   } else{
+    if(sum(chain_n(sampler)) == 0) stop("Run fit to collect MCMC samples before using this function")
     idx <- which(sampler[[1]]$samples$stage %in% stage)
   }
   if(length(idx) == 0) stop("Make sure there are already samples of the selected stage")
@@ -215,7 +226,7 @@ get_objects_blocked <- function(selection, sample_prior, return_prior, design =
       Sigma = 'Group-level covariance matrix'
     )
     if(!is.null(list(...)$return_info)) return(prior[c("types", "type_descriptions", "descriptions")])
-    prior$prior <- add_prior_names(prior$prior, design)
+    prior$prior <- add_prior_names(prior$prior, design, ...)
     return(prior)
   } else{
     if(!selection %in% acc_selection) stop(paste0("selection must be in : ", paste(acc_selection, collapse = ", ")))
@@ -234,7 +245,8 @@ get_objects_blocked <- function(selection, sample_prior, return_prior, design =
                                                c(list(prior = prior, design = design,
                                               selection = selection,N = N), fix_dots(dots, get_prior_blocked)))))
       }
-      attr(sampler, "design_list") <- list(design)
+      sampler[[1]]$prior <- prior
+      class(sampler) <- "emc"
       return(sampler)
     }
     idx <- get_idx(sampler, stage)
@@ -260,13 +272,14 @@ get_objects_single <- function(selection, sample_prior, return_prior, design = N
       alpha = "Subject-level prior"
     )
     if(!is.null(list(...)$return_info)) return(prior[c("types", "type_descriptions", "descriptions")])
-    prior$prior <- add_prior_names(prior$prior, design)
+    prior$prior <- add_prior_names(prior$prior, design, ...)
     return(prior)
   } else{
     if(!selection %in% acc_selection) stop(paste0("selection must be in : ", paste(acc_selection, collapse = ", ")))
     if(sample_prior){
       sampler <- list(list(samples = get_prior_single(prior = prior, design = design, selection = selection,N = N)))
-      attr(sampler, "design_list") <- list(design)
+      sampler[[1]]$prior <- prior
+      class(sampler) <- "emc"
       return(sampler)
     }
     idx <- get_idx(sampler, stage)
@@ -299,7 +312,7 @@ get_objects_factor <- function(selection, sample_prior, return_prior, design = N
       residuals = "Residual errors on the variances"
     )
     if(!is.null(list(...)$return_info)) return(prior[c("types", "type_descriptions", "descriptions")])
-    prior$prior <- add_prior_names(prior$prior, design)
+    prior$prior <- add_prior_names(prior$prior, design, ...)
     return(prior)
   } else{
     if(!selection %in% acc_selection) stop(paste0("selection must be in : ", paste(acc_selection, collapse = ", ")))
@@ -318,7 +331,8 @@ get_objects_factor <- function(selection, sample_prior, return_prior, design = N
                                                c(list(prior = prior, design = design,
                                                       selection = selection,N = N), fix_dots(dots, get_prior_factor)))))
       }
-      attr(sampler, "design_list") <- list(design)
+      sampler[[1]]$prior <- prior
+      class(sampler) <- "emc"
       return(sampler)
     }
     idx <- get_idx(sampler, stage)
@@ -361,7 +375,7 @@ get_objects_infnt_factor <- function(selection, sample_prior, return_prior, desi
       residuals = "Residual errors on the variances"
     )
     if(!is.null(list(...)$return_info)) return(prior[c("types", "type_descriptions", "descriptions")])
-    prior$prior <- add_prior_names(prior$prior, design)
+    prior$prior <- add_prior_names(prior$prior, design, ...)
     return(prior)
   } else{
     if(!selection %in% acc_selection) stop(paste0("selection must be in : ", paste(acc_selection, collapse = ", ")))
@@ -376,7 +390,8 @@ get_objects_infnt_factor <- function(selection, sample_prior, return_prior, desi
                                                c(list(prior = prior, design = design,
                                                       selection = selection,N = N), fix_dots(list(...), get_prior_infnt_factor)))))
       }
-      attr(sampler, "design_list") <- list(design)
+      sampler[[1]]$prior <- prior
+      class(sampler) <- "emc"
       return(sampler)
     }
     idx <- get_idx(sampler, stage)
@@ -429,7 +444,7 @@ get_objects_SEM <- function(selection, sample_prior, return_prior, design = NULL
       structural_regressors = "structural regressors between factors"
     )
     if(!is.null(list(...)$return_info)) return(prior[c("types", "type_descriptions", "descriptions")])
-    prior$prior <- add_prior_names(prior$prior, design)
+    prior$prior <- add_prior_names(prior$prior, design, ...)
     return(prior)
   } else{
     if(!selection %in% acc_selection) stop(paste0("selection must be in : ", paste(acc_selection, collapse = ", ")))
@@ -450,7 +465,8 @@ get_objects_SEM <- function(selection, sample_prior, return_prior, design = NULL
                                                c(list(prior = prior, design = design,
                                                       selection = selection,N = N), fix_dots(dots, get_prior_SEM)))))
       }
-      attr(sampler, "design_list") <- list(design)
+      sampler[[1]]$prior <- prior
+      class(sampler) <- "emc"
       return(sampler)
     }
     idx <- get_idx(sampler, stage)
